##############################################################################
# TAG HEBREW AND GREEK LANGUAGE IN ASCII FILES
##############################################################################
#
# (c) 2004 www.SpiritAndTruth.org
# May be freely used without restriction.
# mailto:contact@SpiritAndTruth.org
#
#

HELP = '''
Usage: langtag [flags] path1 [path2...]

FLAGS
-----

-?          Print out help and exit.

-help       Print out help and exit.

-advance    Normally, breathmarks are only advanced to precede
            capital vowels. When the -advance flag is specified,
            breathmarks for all vowels and diphthongs are advanced
            to precede the vowel or diphthong.

-space      When advancing breathmarks (see -advance), insert
            an extra space ahead of the advanced breathmark to
            increase separation from the preceding text.

-unicode    Use unicode fonts (see below).

TYPICAL USAGE
-------------
The most common way which langtag is used is to tag HTML 
files containing Hebrew or Greek language represented as ASCII
characters and convert them for display using HTML markup.

Hebrew and Greek input appears within curly braces in either of two styles:

    Greek style 1: {grk|Ijerosovluma}. 
    Greek style 2: {g Ijerosovluma}. 

    Hebrew style 1: {heb|yis.r'ael}.
    Hebrew style 2: {h yis.r'ael}.

Note that an initial "h " or "heb|" designates Hebrew and an initial "g " or
"grk|" designates Greek characters to follow within the braces.
(Why two styles? For backward compatibility with texts previously developed on
our website.)

The codes used for the Hebrew and Greek input text correspond to the free
BSTHebrew and BSTGreek fonts available from

    https://www.SpiritAndTruth.org/fonts/index.htm

The Hebrew or Greek markup is automatically converted as follows:

1. Final characters (e.g, sigma in Greek, mem in Hebrew) are
   converted to the appropriate form. (This means you can type a
   final 's' or 'm' in your language and let the software take
   care of using the alternative final form.)

2. Appropriate <span> markers are placed around the text.
   For example:

        <span class="greek">Ijerosuvluma</span> (Jerusalem)
        <span class="hebrew">yis.r'ael</span> (Israel)

   The user must use an associated style (usually from a style sheet)
   to associate a font and other display characteristics with each
   language (see below).

3. If the -unicode flag is NOT specified, then Hebrew text is
   automatically reversed:

        <span class="hebrew">lea'r.siy</span> (Israel)

4. If the -unicode flag is specified, then the order of the Hebrew text
   is not modified. Applications, such as Web browsers, which support unicode
   automatically reverse the order of Hebrew when they display the text.

   In addition, the -unicode flag causes the class names within the <span>
   markers designating Greek and Hebrew include a "u" prefix:

        <span class="ugreek">Ijerosuvluma</span> (Jerusalem)
        <span class="uhebrew">yis.r'ael</span> (Israel)

    This allows the same style sheet to specify different fonts for use with
    non-unicode vs. unicode fonts.

    You can use the included style sheet by placing the following line
    in the header portion of your HTML file:

        <link rel=stylesheet type="text/css" HREF="styles.css">


    Or you can specify the styles in the <head> portion of the document:

        <head>
            ...
            <style> 
            <!-- 
            .ugreek
                {
                font-family:BibliaLS;
                color:blue;
                font-size:130%;
                }
            .greek
                {
                font-family:BSTGreek;
                color:blue;
                font-size:130%;
                }
            .uhebrew
                {
                font-family:BibliaLS;
                color:blue;
                font-size:150%;
                }
            .hebrew
                {
                font-family:BSTHebrew;
                color:blue;
                font-size:150%;
                }
            -->
            </style> 
        </head>

    See the associated styles.css file as an example.

CONVERTING FILES
----------------
A command-line (DOS) utility named LANGTAG.EXE is included. To convert one or
more files, this utility can be run at the command line to specify the flags
and files (and/or folders) to convert. Alternately, if the default operation
is adequate, you can drag and drop files to convert them:

1. Within Explorer, drag-and-drop the input file(s) onto the LANGTAG.EXE
   file in the directory where LANGTAG.EXE was unzipped.

2. LANGTAG.EXE will create a corresponding output file in the same 
   folder as the original input file, but with a different name:

   Dropping 'MyFile.htm' will create 'MyFile_langtag.htm' as the translated
   output file.

Developed by: www.SpiritAndTruth.org contact@SpiritAndTruth.org
'''

VERSION = '0.1'
HISTORY = '''

HISTORY

    0.1     Initial version.
'''

import hebgrk
import os.path
import sys
import string
import re

##############################################################################
# Visit a directory when walking a directory tree.
##############################################################################
def visit( processor, dirname, names ) :
    # Visit each directory below a directory tree.

    #print 'Visiting directory:', dirname
    for name in names :
        processor.scanFile( os.path.join( dirname, name ) )

    return


##############################################################################
# Identify and tag language markup within files.
##############################################################################
class Processor :


    def __init__( self, args ) :
        self.m_me = args.pop( 0 )
        self.m_langPat = '{(?P<id>(h|g)\s+|(heb|grk)\|)(?P<text>[^}]+)}'
        self.m_langExp = re.compile( self.m_langPat, re.IGNORECASE )
        self.m_unicode = 0
        self.m_advance = 0
        self.m_space = 0
        self.m_spanFormatGreek = '<span class="greek">%s</span>'
        self.m_spanFormatHebrew = '<span class="hebrew">%s</span>'

        # Help requested?
        try :
            idx = args.remove( '-?' )
            print VERSION
            print HELP
            print HISTORY
            sys.exit( 1 )
        except ValueError : pass
        try :
            idx = args.remove( '-help' )
            print VERSION
            print HELP
            print HISTORY
            sys.exit( 1 )
        except ValueError : pass

        # Unicode specified?
        try :
            idx = args.remove( '-unicode' )
            self.m_unicode = 1
            print 'Using unicode'
            self.m_spanFormatGreek = '<span class="ugreek">%s</span>'
            self.m_spanFormatHebrew = '<span class="uhebrew">%s</span>'
        except ValueError : pass

        # Advance breath marks?
        try :
            idx = args.remove( '-advance' )
            self.m_advance = 1
            print 'Advancing breathmarks'
        except ValueError : pass

        # Space when advance breathmarks?
        try :
            idx = args.remove( '-space' )
            self.m_space = 1
        except ValueError : pass

        # For each name to be scanned for tags.
        for path in args :

            # Walk directories.
            if os.path.isdir( path ) : 
                os.path.walk( path, visit, self )

            # Scan individual files.
            elif os.path.isfile( path ) : 
                self.scanFile( path )


    def scanFile( self, path ) :
        # Scan file and tag for language.
        inFile = None
        outFile = None

        try :

            # Open the file for read.
            print
            print 'Reading', path
            inFile = open( path, 'r' )

            # Open a corresponding output file.
            file, ext = os.path.splitext( path )
            outPath = file + '_langtag' + ext
            print 'Writing', outPath
            outFile = open( outPath, 'w' )

            # For each input line.
            for line in inFile.readlines() :

                # Translate for corresponding output.
                outFile.write( self.translateLanguage( line ) )

        finally :
            if inFile : inFile.close()
            if outFile : outFile.close()


    def translateLanguage( self, line ) :
        # Translate a line of language markup.

        # Forever.
        output = ''
        remainder = line
        while 1 :

            # If found another language markup in the line.
            match = self.m_langExp.search( remainder )
            if match :

                # Pass through portion up to the markup.
                output += remainder[ 0 : match.start() ]

                # Get the language text.
                language = match.group( 'text' )

                # If it is Hebrew.
                id = match.group( 'id' )
                if 'h' == id[ 0 ].lower() :

                    print 'Hebrew:', language

                    #Convert finals.
                    language = hebgrk.hebrewFinals( language )

                    # If using unicode.
                    if self.m_unicode :

                        # Convert to unicode, application will take care of reverse display.
                        language = hebgrk.BSTHebrewToUnicodeHtml( language, 0, not self.m_unicode )

                    # Else not using unicode.
                    else :

                        # Reverse Hebrew text.
                        language = list( language )
                        language.reverse()
                        language = string.join( language, '' )

                    # Tag with <span>
                    output += self.m_spanFormatHebrew % language

                # Else it is Greek.
                else :

                    print 'Greek:', language

                    # Convert finals.
                    language = hebgrk.greekFinals( language )

                    # Advance breathmarks?
                    language = hebgrk.advanceBreathMarkCapital( language, self.m_space )
                    if self.m_advance :
                        language = hebgrk.advanceBreathMark( language, self.m_space )

                    # Translate Greek to unicode?
                    if self.m_unicode : 
                        language = hebgrk.BSTGreekToUnicodeHtml( language )

                    # Tag with <span>
                    output += self.m_spanFormatGreek % language

                # Move past the language markup.
                remainder = remainder[ match.end() : ]

            else :
                break

        return output + remainder

# If module is launched as a program, perform tagging using specified files.
if __name__ == '__main__' :
    tag = Processor( sys.argv )
    
